# -------------------------------------------------------------------------------- NOTEBOOK-CELL: CODE
#%config Completer.use_jedi = False

# -*- coding: utf-8 -*-
import dataiku
import pandas as pd, numpy as np
from dataiku import pandasutils as pdu
import pm4py
from data_management import get_managed_folder_id_with_folder_name, read_pickle_from_dss_folder
from pm4py.algo.conformance.tokenreplay import algorithm as token_replay
from collections import OrderedDict

# Read recipe inputs
workflow_clean = dataiku.Dataset("workflow_clean")
workflow_clean_df = workflow_clean.get_dataframe()

event_log = pm4py.format_dataframe(workflow_clean_df, case_id='case', activity_key='activity', timestamp_key='timestamp')

# -------------------------------------------------------------------------------- NOTEBOOK-CELL: CODE
input_folder = dataiku.Folder('1rhVX912')
paths = input_folder.list_paths_in_partition()
processes = [name.replace('_petri.p', '').replace('/', '') for name in paths if '_petri.p' in name]
petri_dicts = {process: read_pickle_from_dss_folder(process + '_petri.p', '1rhVX912') for process in processes}

# -------------------------------------------------------------------------------- NOTEBOOK-CELL: CODE
conformance_checks_df = pd.DataFrame()

case_numbers = list(OrderedDict.fromkeys(event_log['case:concept:name']))

for process in petri_dicts.keys():
    petri_dict = petri_dicts.get(process)
    net = petri_dict['pn']
    im = petri_dict['im']
    fm = petri_dict['fm']
    traces = pd.DataFrame(token_replay.apply(event_log, net, im, fm))
    traces['reference_process'] = process
    traces['case'] = case_numbers
    conformance_checks_df = conformance_checks_df.append(traces)

# -------------------------------------------------------------------------------- NOTEBOOK-CELL: CODE
base_columns = ['case', 'activity', 'timestamp']
additional_columns = [col for col in workflow_clean_df.columns if col not in base_columns]

workflow_start_times = workflow_clean_df.groupby(['case'])['timestamp'].min().reset_index()
workflow_traces = workflow_clean_df.groupby(['case'])[additional_columns].last().reset_index()
workflow_info = pd.merge(workflow_traces, workflow_start_times, on='case')
workflow_info['case'] = workflow_info['case'].apply(str)
conformance_checks_df['case'] = conformance_checks_df['case'].apply(str)
conformance_checks_enriched_df = pd.merge(conformance_checks_df, workflow_info, how='left', on='case')

# Write recipe outputs
conformance_checks = dataiku.Dataset("process_conformance_checks")
conformance_checks.write_with_schema(conformance_checks_enriched_df)